//===========================================================================
//
//      Module: CPPAPI.CPP
//
//      Description:
//
//          This sample application demonstrates interoperability between
//          the Notes/Domino C++ API and the Domino Objects for COM.
//          Both COM and C++ API environments can be initialized
//          concurrently within the same process with no conflicts.
//
//          The COM code takes care of a problem with the C++ API that
//          makes it very hard or impossible to pass in the user's password
//          programmatically to a Notes Session.  It also gets the name of
//          the Database and prints it to a Windows Message box, like the
//          intro sample does.
//
//          The C++ API part does low level Design Note things that can be
//          done easily using the C++ API but would be extremely difficult
//          or impossible to do using the other toolkits.
//
//      Syntax: CPPAPI <user-password>
//
//===========================================================================
//
// Copyright (c) 2000 Lotus Development Corporation. All rights reserved.
// This software is subject to the Lotus Software Agreement, Restricted
// Rights for U.S. government users, and applicable export regulations.
//
//===========================================================================

// Needed for UNICODE support.
#define UNICODE
#define USING_CAPI

#include <iostream.h>

// Domino COM headers.
#include "domobj.h"

// Notes C++ API headers.
#include "lncppapi.h"


char CommandBuf[80];

/*******************************************************************

  Function: main 
  
********************************************************************/
int main(int argc, char *argv[])
{    
    HRESULT         hr;                     // Return Status from COM calls.
    BSTR            bstrPassword = NULL;
    BSTR            bstrUserName = NULL;
    ISession*       pComSession = NULL;     // DT COM session object.
    LNNotesSession  CppSession;             // C++ API session object.
    int             ProgramStatus = 0;
    wchar_t         wcstrBuffer[1024];    
    
	// Begin Try block. (Set Throw all errors macro for C++ API)
	LNSetThrowAllErrors (TRUE);

	try
	{
        // Initialize and set up COM environment.       
        hr = CoInitialize(NULL);
        if (FAILED(hr))
            throw "CPPAPI ERROR: Failed to initialize the COM environment";

        // Instantiate NotesSession interface for DT COM.
        hr = CoCreateInstance( CLSID_NotesSession,
                                          NULL,
                                          CLSCTX_SERVER,
                                          IID_ISession,
                                          (LPVOID*)&pComSession );
        if (FAILED(hr))
            throw "CPPAPI ERROR: Failed to Instantiate COM Notes Session";

        if ( argc == 2 )                    
        {
          // Convert the multibyte string to wide characters.
          mbstowcs(wcstrBuffer, argv[1], strlen(argv[1])+1);

          // Allocate a BSTR for the database name.
          bstrPassword = SysAllocString(wcstrBuffer);

          // Initialize the COM NotesSession. 
          hr = pComSession->Initialize(bstrPassword);
        }
        else
          hr = pComSession->Initialize(NULL);


        if (FAILED(hr))
            throw "CPPAPI ERROR: Failed to Initialize COM Notes Session";       
    
		// Initialize the C++ API NotesSession.
		CppSession.Init (argc, argv);

        // Use COM to get the user name and display it in a message box.
        pComSession->get_UserName(&bstrUserName);
        MessageBox( NULL, bstrUserName, L"Current User Name", MB_OK);

        // Now do some heavy duty C++ API stuff.
        // C++ API Local declarations...

        LNDatabase              db;
        LNCreateDatabaseOptions options;
        LNViewFolder            view;
        LNVFColumn              col;
	    LNForm		            form;
        LNDocument              doc;
        LNText                  title;
        LNRichText              formbody;
        LNRichText              body;
		LNRTCursor			    startcursor;
		LNRTCursor			    endcursor;
		LNFontStyle			    fs;                
        LNString                dbpath;

        // Create a new blank database in the user's Notes data directory.
        // Overwrite an existing database if it exists from before.  
        // Create automatically opens it.  
       
        dbpath << "createdoc.nsf";

		cout << endl << "***** Opening database: " << dbpath << " ******" << endl;

        //options.SetOverwriteExistingFile(TRUE);
        CppSession.GetDatabase(dbpath, &db);

        db.Open();
        
		// Create a new form in the new database.
        // Make this the default form.

		cout << "***** Creating a new form in the database: \"CPPAPI Form\" ******" << endl;

		db.CreateForm( "CPPAPI Form", &form );
        //form.SetIsDefaultForm(TRUE);

        // Create two form fields:  One plain text "Title" field
        // and the other one a rich text "Body" field.
        // Add descriptive text and formatting to the Title field
        // Using LNFontStyle object.(fs)

        form.GetFormBody(&formbody);
		formbody.GetCursor( &startcursor );
		formbody.GetEndCursor( &endcursor );

		fs.SetDefaults();
		fs.SetAttributes( LNFONTATTRIBUTEFLAGS_PLAIN );
		fs.SetFaceID( LNFACEID_ROMAN );
		fs.SetColor( LNCOLOR_DARK_BLUE );
		fs.SetPointSize( 24 );

        formbody.Insert("Document Title: ", &endcursor);
        formbody.CreateFormField("Title", LNRTTYPE_TEXT_FIELD, &endcursor);
		formbody.SetFontStyle( startcursor, endcursor, fs );

        formbody.StartNewParagraph(&endcursor);
        formbody.CreateFormField("Body",   LNRTTYPE_RICH_TEXT_FIELD, &endcursor);

		// Save, Close the new form.
		form.Save();
		form.Close();

        // Create a new document in the database.
        // It will use the default form we just created.

		cout << "***** Creating a new document in the database. ******" << endl;

        db.CreateDocument(&doc);

        // Give the doc a title, and add some text to the document's body.

        title.SetValue("Yard sale on Sunday");
        doc.CreateItem("Title", title, LNITEMFLAGS_SUMMARY, LNITEMOPTION_DELETE_APPEND);

        body.SetValue("Some items for sale: Chairs, Tables, Pictures");
        doc.CreateItem("Body", body, LNITEMFLAGS_NONE, LNITEMOPTION_DELETE_APPEND);

        doc.SetForm("CPPAPI Form");

        // Save and close the doc.

        doc.Save();
        doc.Close();

		// Close the database, we are all done now...

		db.Close();

		ProgramStatus = 0;

	} // END try

    catch (LNSTATUS lnerror)
    {
        char ErrorBuf[LNERROR_MESSAGE_LENGTH];

        LNGetErrorMessage(lnerror, ErrorBuf, LNERROR_MESSAGE_LENGTH);
        cout << "\nException thrown during execution of main process." << endl;
        cout << "\nError:  " << ErrorBuf << endl;
        ProgramStatus = 3;
    }

    catch (const char *pErrorMessage)
    {
        cout << "\nException thrown during execution of main process." << endl;
        cout << "\nError: " << pErrorMessage << endl << endl;
        ProgramStatus = 2;
    }

    catch (...)
    {
        cout << "\nException thrown during execution of main process." << endl;
        cout << "\nError unknown!" << endl;
        ProgramStatus = 1;
    }

    char CommandBuf[80];
    cout << "All Done.  Hit return to exit: ";
    cin.getline(CommandBuf, 80);

    // Free the BSTR's.
    if (bstrPassword) SysFreeString(bstrPassword); 
    if (bstrUserName) SysFreeString(bstrUserName);   
    
    // Release the COM NotesSession.
    if (pComSession) pComSession->Release();

    // Close the COM Library and free resource.
    CoUninitialize();	

    // Terminate the C++ API.
    CppSession.Term();

    return (ProgramStatus);

} // END MAIN


