/*******************************************************************

  File:   	CreateDoc.cpp 

  This program reads from comma separated file and creates a 
  new document for in the customer database (CreateDoc.nsf)
  for each line.

*******************************************************************/

#define UNICODE 
       
#include <windows.h>
#include <stdio.h>
#include <fstream.h>
#include <io.h>     

#include "domobj.h"

// Globals array of wide character strings.
const static wchar_t* wcItemNames[]=
{
   L"F_FirstName",
   L"F_LastName",
   L"F_StreetAddress",         
   L"F_City",
   L"F_State",
   L"F_ZipCode",
   L"F_Phone"
};     

// Function Prototypes. 
void ReadCSVFile(IDatabase* pDatabase);
void CreateItem(IDocument* pDocument, const wchar_t* lpwcItemName, const wchar_t* lpwcItemValue);
void PrintCOMError(IUnknown* pIObject, IID IID_IObjectInterface);

/*******************************************************************

  Function: main 
  
********************************************************************/
      
void main()
{
   // Return Status from COM calls.
   HRESULT hr;
  
   // Domino Objects
   ISession* pSession = NULL;  
   IDatabase* pDatabase = NULL;

   BSTR bstrServerName = NULL;  
   BSTR bstrDatabaseName = NULL;  
   BSTR bstrDBTitle = NULL;
       
   // Initialize COM Library.
   hr = CoInitialize(NULL);
   if (FAILED(hr))
   {
      return;
   }

   // Instantiate NotesSession interface.
   hr = CoCreateInstance(
      CLSID_NotesSession,
      NULL,
      CLSCTX_SERVER,
      IID_ISession, 
      (LPVOID*)&pSession);
   if (FAILED(hr))
   {
      goto Error;
   }
   
   // Initialize the NotesSession.    
   hr = pSession->Initialize(NULL);
   if (FAILED(hr))
   {
      PrintCOMError(pSession, IID_ISession);
      goto Error;
   }

   // Allocate a BSTR for the database name.
   bstrDatabaseName = SysAllocString( L"CreateDoc.nsf");
   
   // Get the database.
   hr = pSession->GetDatabase(
       bstrServerName, 
       bstrDatabaseName, 
       FALSE, 
       &pDatabase);
   if (FAILED(hr))
   {
      PrintCOMError(pDatabase, IID_IDatabase);
      goto Error;
   }

   if (pDatabase == NULL)
   {
      wprintf(L"CreateDoc.nsf does note exist in the Notes data directory.\n");
      wprintf(L"Please copy this file into the Notes data directory.");  
      goto Error;
   }

   // Get the title.
   pDatabase->get_Title(&bstrDBTitle);  

   // Print the title.
   wprintf(L"Opened Database : %s\n", bstrDBTitle);

   // Open and parse the input file.
   ReadCSVFile(pDatabase);         
   
Error:
   
   // Free the BSTR.
   if (bstrDBTitle) SysFreeString(bstrDBTitle);

   // Release the NotesSession.
   if (pSession) pSession->Release();

   // Close the COM Library and free resource.
   CoUninitialize();	

   // Print the title.
   wprintf(L"Done\n");
}

/*******************************************************************

  Function: ReadCSVFile 

  Purpose:  Opens and parses a CSV file called customer.csv. A 
            new document is created for each line in the file.
  
********************************************************************/

void ReadCSVFile(IDatabase* pDatabase)
{ 
   // Return Status from COM calls.
   HRESULT hr;  

   // Allocate some buffers.
   char szBuffer[1024];
   wchar_t wcstrBuffer[1024];

   short sResult;  
   
   // Open the customer file.
   ifstream is( "customer.csv", ios::in | ios::nocreate );
   if( !is ) 
   {
      cout << "ERROR: Cannot open file 'customer.csv'." << endl;
      return;
   }

   // While not at end of file.
   while(is.peek() != EOF)
   {
      // Domino Objects.
      IDocument* pDocument = NULL;

      // Get a line of data.
      is.getline(szBuffer, 1024);

      // Create a new document for each line.
      hr = pDatabase->CreateDocument(&pDocument);
      if (FAILED(hr))
      {
         PrintCOMError(pDatabase, IID_IDatabase);
         cout << "ERROR: Cannot create document." << endl; 
         return;
      }
      cout << "Adding a new customer to database." << endl;
      cout << szBuffer << endl;     
      
      // Lenght of the line of data.
      int iLength = strlen(szBuffer);

      // Count of items processed.
      int iItem = 0;

      // Second pointer to the line of data. 
      char* lpszTemp = szBuffer;

      // Parse the data.
      for(int i=0; i<=iLength; ++i)
      {   
         // Each comma or null represents an item delimter.
         if(szBuffer[i] == ',' || szBuffer[i] == '\0')
         {
            // Replace all commas with a null.
            szBuffer[i] = '\0';
            
            // Conver the multibyte string to wide characters.
            mbstowcs(wcstrBuffer, lpszTemp, strlen(lpszTemp)+1);

            // Create a new item for each delimiter found.
            CreateItem(pDocument, wcItemNames[iItem], wcstrBuffer);

            // Move to the next delimiter.
            lpszTemp = &szBuffer[i];

            // Bump the pointer to the first character after the delimiter.
            ++lpszTemp;

            // Increment the item count.
            ++iItem;
         }  
      }// for

      // Create an item called Form. 
      // This will be used to display the document.
      CreateItem(pDocument, L"Form", L"Customer");
      
      // Save the new document.
      hr = pDocument->Save(FALSE, FALSE, FALSE, &sResult);
      if (FAILED(hr))
      {
         PrintCOMError(pDatabase, IID_IDatabase);
         cout << "ERROR: Cannot save document." << endl; 
         return;
      }

      // Release the NotesDocument.
      if (pDocument) pDocument->Release();

   } // while        

}

/*******************************************************************

  Function: CreateItem 

  Purpose:  Creates a new item in a document. The name and value
            of the item are passed in as pointers to wide character
            strings. 
  
********************************************************************/

void CreateItem(IDocument* pDocument, const wchar_t* lpwcItemName, const wchar_t* lpwcItemValue)
{
   // Domino Objects
   IItem* pItem = NULL;

   BSTR bstrItemName = NULL;  
   BSTR bstrItemValue = NULL;
   
   VARIANT varItemValue; 

   // Allocate a BSTR's.
   bstrItemName = SysAllocString(lpwcItemName);
   bstrItemValue = SysAllocString(lpwcItemValue);

   // Fill the variant.
   varItemValue.vt = VT_BSTR;
   varItemValue.bstrVal = bstrItemValue;

   // Add an item to the document.
   pDocument->AppendItemValue(bstrItemName, varItemValue, &pItem);

   // Free the BSTR's.
   if (bstrItemName) SysFreeString(bstrItemName);      
   if (bstrItemValue) SysFreeString(bstrItemValue);   

   // Release the NotesItem.
   if (pItem) pItem->Release();
}

/*******************************************************************

  Function: PrintCOMError

  Purpose:  Prints the COM error.
  
********************************************************************/
void PrintCOMError(IUnknown* pIObject, IID IID_IObjectInterface)
{
   HRESULT hr;
   ISupportErrorInfo *pSupportErrorInfo = 0;
  
   hr = pIObject->QueryInterface( IID_ISupportErrorInfo,
                                 (void**)&pSupportErrorInfo );
   if( SUCCEEDED(hr)) {
      hr = pSupportErrorInfo->InterfaceSupportsErrorInfo(IID_IObjectInterface);
      if ( hr == S_OK ) {
         IErrorInfo *pErrorInfo = NULL;
         hr = GetErrorInfo(0, &pErrorInfo);
         if ( hr == S_OK ) {

            BSTR bstrSource = NULL;
            BSTR bstrDescription = NULL;

            hr = pErrorInfo->GetSource(&bstrSource);
            hr = pErrorInfo->GetDescription(&bstrDescription);
            
            // Print the error.
            wprintf(L"COM Error : %s", bstrDescription);

            // Free the BSTR's.
            if (bstrSource) SysFreeString(bstrSource);
            if (bstrDescription) SysFreeString(bstrDescription);
         }
      }
      if (pSupportErrorInfo) pSupportErrorInfo->Release();
   }
}


