{==============================================================================|
| Project : Notes/Delphi class library                           | 3.8         |
|==============================================================================|
| Content:                                                                     |
|==============================================================================|
| The contents of this file are subject to the Mozilla Public License Ver. 1.0 |
| (the "License"); you may not use this file except in compliance with the     |
| License. You may obtain a copy of the License at http://www.mozilla.org/MPL/ |
|                                                                              |
| Software distributed under the License is distributed on an "AS IS" basis,   |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for |
| the specific language governing rights and limitations under the License.    |
|==============================================================================|
| Initial Developers of the Original Code are:                                 |
|   Sergey Kolchin (Russia) skolchin@usa.net ICQ#2292387                       |
|   Sergey Kucherov (Russia)                                                   |
|   Sergey Okorochkov (Russia)                                                 |
| All Rights Reserved.                                                         |
|   Last Modified:                                                             |
|     17.11.99, Sergey Kolchin                                                 |
|==============================================================================|
| Contributors and Bug Corrections:                                            |
|   Fujio Kurose                                                               |
|   Noah Silva                                                                 |
|   Tibor Egressi                                                              |
|   Andreas Pape                                                               |
|   Anatoly Ivkov                                                              |
|   Winalot                                                                    |
|     and others...                                                            |
|==============================================================================|
| History: see README.TXT                                                      |
|==============================================================================|
| Notes password operations                                                    |
|                                                                              |
| The unit contains functions used to intercept Notes password prompt and      |
| provide a custom password to it                                              |
| LNPASS.DLL 2.1 must be available either in the current or in the Notes exe   |
| directory. To (un)install it, use InstallLnPass/UninstallLnPass              |
| functions or modify NOTES.INI by adding the line:                            |
|        EXTMGR_ADDINS=lnpass.dll                                              |
|                                                                              |
| WARNING!                                                                     |
| Please note that the functions do not attempt to encrypt passwords as Notes  |
| do. Using them can compromise Notes security, so do not use them in programs |
| which communicate with user - let Notes do the job. The functions are        |
| designed for server applications which must run without user interventions   |
|==============================================================================|}

unit Util_LnPass;

interface

{$IFNDEF WIN32}
-- This unit is for Windows 32 environment
{$ENDIF}
{$RANGECHECKS OFF}
{$ALIGN OFF}

uses SysUtils, Classes, Windows, Class_LotusNotes, Util_LnAPIErr, Util_LnApi;

// LNPASS installation/uninstallation
// If LnPassPath = '', function searches for it in the current directory
// If LNPASS is not in Notes exe directory, the function COPIES it to there
function InstallLnPass(LnPassPath: string): boolean;
procedure UninstallLnPass;

// Set password
// If fCheck is True, the function attempts to perform Notes operation which
// requires a password and returns FALSE if it's incorrect.

// Otherwise, the function doesn't check password and if it is wrong,
// Notes function which requires a password will fail with the exception
// ELotusNotes with ErrorCode = ERR_SECURE_BADPASSWORD.
function SetPassword(UserName, Password: string; fCheck: boolean): boolean;

// Clear the password set by previous function
procedure ClearPassword;

const
  LNPASS_DLL_NAME = 'LNPASS.DLL';
  EXTMGR_VAR      = 'EXTMGR_ADDINS';

implementation
uses Class_NotesRTF;

(******************************************************************************)
function InstallLnPass;
var
  DataDir: string;
  EnvStr: string;
begin
  // Make sure LNPASS in Notes directory
  DataDir := GetNotesDataDir();
  if (DataDir <> '') and (DataDir[length(DataDir)] <> '\') then appendStr(DataDir,'\');

  if LnPassPath = '' then begin
    // check LNPASS in data dir
    LnPassPath := DataDir + LNPASS_DLL_NAME;
    Result := FileExists(LnPassPath);
    if not Result then begin
      // check LNPASS in current directory
      LnPassPath := extractFilePath(paramStr(0)) + LNPASS_DLL_NAME;
      Result := FileExists(LnPassPath);
      if not Result then exit;    //cannot find LNPASS, exiting
    end;
  end;
  if compareText(extractFilePath(LnPassPath), DataDir) <> 0 then begin
    // Copy LNPASS to data directory
    CopyFile(pchar(LnPassPath), pchar(DataDir + LNPASS_DLL_NAME), False);
  end;
  Result := FileExists(DataDir + LNPASS_DLL_NAME);
  if not Result then exit;

  // Get environ string
  EnvStr := '';
  setLength(EnvStr, 255);
  OsGetEnvironmentString(EXTMGR_VAR, pchar(EnvStr), 254);
  EnvStr := trim(strPas(pchar(EnvStr)));

  // Check for already installed add-in
  if Pos (LNPASS_DLL_NAME, UpperCase(EnvStr)) = 0 then begin
    // Not found, add the name
    if EnvStr = ''
      then EnvStr := LNPASS_DLL_NAME
      else appendStr(EnvStr, ',' + LNPASS_DLL_NAME);

    // Store changed line
    OSSetEnvironmentVariable(EXTMGR_VAR, pchar(EnvStr));
  end;
end;

(******************************************************************************)
procedure UninstallLnPass;
var
  EnvStr, NewStr: string;
  i, ofs, len, cofs: integer;
begin
  // Get environ string
  EnvStr := '';
  setLength(EnvStr, 255);
  OsGetEnvironmentString(EXTMGR_VAR, pchar(EnvStr), 254);
  EnvStr := trim(strPas(pchar(EnvStr)));

  // Find location
  ofs := Pos (LNPASS_DLL_NAME, UpperCase(EnvStr));
  if ofs = 0 then exit;

  // Find next comma or end of line
  len := length(EnvStr);
  cofs := 0;
  for i := ofs + 1 to len do begin
    if EnvStr[i] = ',' then begin
      cofs := i;
      break;
    end;
  end;

  // Y.Kitamura: if EOLN, remove all preceding commas and spaces
  if (ofs > 1) and (cofs = 0) then begin
    while (ofs > 0) and ((EnvStr[ofs-1] = ' ') or (EnvStr[ofs-1] = ',')) do begin
      ofs := ofs-1;
    end;
  end;
  
  NewStr := '';
  if ofs > 1 then NewStr := copy(EnvStr,1,ofs-1);
  if cofs > 0 then appendStr (NewStr, copy(EnvStr,cofs + 1, len));

  OSSetEnvironmentVariable(EXTMGR_VAR, pchar(NewStr));
end;

(******************************************************************************)
threadvar hMapFile: THandle;
const MEMFILE = 'LNPASS.PWD';

const
  REGIDGetPrivateKey = 9;

function REGGetIDInfo(
  IDFileName: pchar;
  InfoType: WORD;
  OutBufr: pointer;
  OutBufrLen: WORD;
  ActualLen: PWORD): STATUS; stdcall; external 'NNOTES.DLL';

(******************************************************************************)
function SetPassword;
var
  ptr: pchar;
  error: STATUS;
  idfile: string;
  sz: WORD;
begin
  if hMapFile = 0 then begin
    hMapFile := CreateFileMapping($FFFFFFFF,nil, PAGE_READWRITE, 0, 255, MEMFILE);
    if hMapFile = 0 then raise EInOutError.Create('Cannot create a memory file');
  end;
  try
    ptr := MapViewOfFile(hMapFile, FILE_MAP_WRITE, 0, 0, 0);
    try
      if ptr <> nil then strCopy(ptr, pchar(Password));
      FlushViewOfFile(ptr, 0);
    finally
      UnMapViewOfFile(ptr);
    end;
  except
    CloseHandle(hMapFile);
    raise;
  end;

  Result := True;
  error := 0;
  if fCheck then try
    setLength(idfile, 256);
    OSGetEnvironmentString('KeyFilename',pchar(idfile),255);
    idfile := strPas(pchar(idfile));
    error := REGGetIDInfo(pchar(idfile),REGIDGetPrivateKey,nil,0,@sz);
  finally
    if error > 0 then begin
      ClearPassword;
      if error = ERR_SECURE_BADPASSWORD then Result := False else CheckError(error);
    end;
  end;
end;

(******************************************************************************)
procedure ClearPassword;
begin
  if hMapFile <> 0 then begin
    CloseHandle(hMapFile);
    hMapFile := 0;
  end;
end;


end.
